;; Generated package description from denote-markdown.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "denote-markdown" "0.2.0" "Extensions that better integrate Denote with Markdown" '((emacs "28.1") (denote "4.0.0")) :commit "588729d70ba118c0ac837cbad756ba05f276afdd" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Protesilaos Stavrou" . "info@protesilaos.com") :url "https://github.com/protesilaos/denote-markdown")
